/**
 * 获取某个 Cookie
 *
 * @param name 键名称
 * @returns 值
 */
function getCookie(name) {
    let arr, reg = new RegExp("(^| )" + name + "=([^;]*)(;|$)");
    // @ts-ignore
    if (arr = document.cookie.match(reg))
        return unescape(arr[2]);
    else
        return null;
}
/**
 * 设置某个 Cookie
 *
 * @param name  键名称
 * @param value 值
 */
function setCookie(name, value) {
    let days = 2, exp = new Date();
    exp.setTime(exp.getTime() + days * 24 * 60 * 60 * 1000);
    // @ts-ignore
    document.cookie = name + "=" + escape(value) + ";path=/;expires=" + exp.toGMTString();
}
/**
 * 清空 Cookie
 */
function delCookie() {
    let keys = document.cookie.match(/[^ =;]+(?==)/g);
    if (keys) {
        let date = new Date(0).toUTCString();
        for (let i = keys.length; i--;) {
            document.cookie = keys[i] + '=0;path=/;expires=' + date; // 清除当前域名下的,例如：m.ratingdog.cn
            document.cookie = keys[i] + '=0;path=/;domain=' + document.domain + ';expires=' + date; // 清除当前域名下的，例如 .m.ratingdog.cn
            document.cookie = keys[i] + '=0;path=/;domain=' + location.host + ';expires=' + date; // 清除一级域名下的或指定的，例如 .ratingdog.cn
        }
    }
}

var cookies = /*#__PURE__*/Object.freeze({
    __proto__: null,
    delCookie: delCookie,
    getCookie: getCookie,
    setCookie: setCookie
});

/**
 * 向父级元素递归搜索
 *
 * @param _el       当前所在元素
 * @param tagName   目标标签名称
 * @param className 目标元素样式类
 * @returns 目标元素，找不到为 null
 */
function up(_el, tagName, className) {
    if (tagName && className)
        throw '只能任选一种参数，不能同时传';
    let el = _el.parentNode;
    tagName = tagName && tagName.toUpperCase();
    while (el) {
        if (tagName && el.tagName == tagName)
            return el;
        if (className && el.className && ~el.className.indexOf(className))
            return el;
        el = el.parentNode;
    }
    return null;
}
/**
 * 加载脚本
 *
 * @param url   脚本地址
 * @param id    脚本元素 id，可选的
 * @param cb    回调函数，可选的
 */
function loadScript(url, id, cb) {
    let script = document.createElement("script");
    script.src = url;
    if (cb)
        script.onload = cb;
    if (id)
        script.id = id;
    document.getElementsByTagName("head")[0].appendChild(script);
}

var dom = /*#__PURE__*/Object.freeze({
    __proto__: null,
    loadScript: loadScript,
    up: up
});

/**
 * 通用工具类
 */
/**
 * 是否调试模式中
 *
 * 打包成组件之后不能用
 *
 * @returns
 */
function isDebug() {
    // @ts-ignore
    return process.env.NODE_ENV === 'development';
}
function isDev() {
    let currentHostname = window.location.hostname;
    // 判断主机名是否是内网地址
    return (currentHostname.startsWith('192.168.') || currentHostname.startsWith('10.') || currentHostname === 'localhost');
}
/**
* 日期格式化。详见博客文章：http://blog.csdn.net/zhangxin09/archive/2011/01/01/6111294.aspx
* e.g: new Date().format("yyyy-MM-dd hh:mm:ss")
*
* @param 	{String} format
* @return	{String}
*/
function dateFormat(format) {
    let $1, o = {
        "M+": this.getMonth() + 1, // 月份，从0开始算
        "d+": this.getDate(), // 日期
        "h+": this.getHours(), // 小时
        "m+": this.getMinutes(), // 分钟
        "s+": this.getSeconds(), // 秒钟
        // 季度 quarter
        "q+": Math.floor((this.getMonth() + 3) / 3),
        "S": this.getMilliseconds() // 千秒
    };
    if (/(y+)/.test(format))
        // @ts-ignore
        $1 = RegExp.$1, format = format.replace($1, String(this.getFullYear()).substr(4 - $1));
    let key, value;
    for (key in o) { // 如果没有指定该参数，则子字符串将延续到 stringvar 的最后。
        if (new RegExp("(" + key + ")").test(format)) {
            $1 = RegExp.$1,
                value = String(o[key]),
                value = $1.length == 1 ? value : ("00" + value).substr(value.length),
                format = format.replace($1, value);
        }
    }
    return format;
}
/**
 * 日期格式化
 * @author meizz
 * @param date  日期，必须为 Date 类型
 * @param fmt   格式模板
 * @returns 格式化后的字符串
 */
function dateFormat2(date, fmt) {
    let o = {
        "M+": date.getMonth() + 1, // 月份   
        "d+": date.getDate(), // 日   
        "h+": date.getHours(), // 小时   
        "m+": date.getMinutes(), // 分   
        "s+": date.getSeconds(), // 秒   
        "q+": Math.floor((date.getMonth() + 3) / 3), // 季度   
        "S": date.getMilliseconds() // 毫秒   
    };
    if (/(y+)/.test(fmt))
        fmt = fmt.replace(RegExp.$1, (date.getFullYear() + "").substr(4 - RegExp.$1.length));
    for (var k in o)
        if (new RegExp("(" + k + ")").test(fmt)) {
            let obj = (RegExp.$1.length == 1) ? o[k] : ("00" + o[k]).substr(("" + o[k]).length);
            // @ts-ignore
            fmt = fmt.replace(RegExp.$1, obj);
        }
    return fmt;
}
/**
 * 并行和串行任务
 *
 * @author https://segmentfault.com/a/1190000013265925
 * @param arr
 * @param finnaly
 */
function parallel(arr, _finally) {
    let fn, index = 0;
    // @ts-ignore
    let statusArr = Array(arr.length).fill().map(() => ({ isActive: false, data: null }));
    let isFinished = function () {
        return statusArr.every((item) => item.isActive === true);
    };
    let resolve = function (index) {
        return function (data) {
            statusArr[index].data = data;
            statusArr[index].isActive = true;
            let isFinish = isFinished();
            if (isFinish) {
                let datas = statusArr.map((item) => item.data);
                _finally(datas);
            }
        };
    };
    // @ts-ignore
    while ((fn = arr.shift())) {
        fn(resolve(index)); // 给 resolve 函数追加参数,可以使用 bind 函数实现,这里使用了柯里化
        index++;
    }
}
/**
 * 函数节流
 *
 * @author https://www.cnblogs.com/moqiutao/p/6875955.html
 * @param fn
 * @param delay
 * @param mustRunDelay
 */
function throttle(fn, delay, mustRunDelay) {
    var timer, t_start;
    return function () {
        var t_curr = +new Date();
        window.clearTimeout(timer);
        if (!t_start)
            t_start = t_curr;
        if (t_curr - t_start >= mustRunDelay) {
            // @ts-ignore
            fn.apply(this, arguments);
            t_start = t_curr;
        }
        else {
            var args = arguments;
            // @ts-ignore
            timer = window.setTimeout(() => fn.apply(this, args), delay);
        }
    };
}
/**
* 复制文字到剪切板
*
* @param {string} text
*/
function copyToClipboard(text) {
    if (navigator.clipboard)
        navigator.clipboard.writeText(text); // clipboard api 复制
    else {
        let textarea = document.createElement('textarea');
        document.body.appendChild(textarea); // 隐藏此输入框
        textarea.style.position = 'fixed';
        textarea.style.clip = 'rect(0 0 0 0)';
        textarea.style.top = '10px';
        textarea.value = text; // 赋值
        textarea.select(); // 选中
        document.execCommand('copy', true); // 复制
        document.body.removeChild(textarea); // 移除输入框
    }
}

var utils = /*#__PURE__*/Object.freeze({
    __proto__: null,
    copyToClipboard: copyToClipboard,
    dateFormat: dateFormat,
    dateFormat2: dateFormat2,
    isDebug: isDebug,
    isDev: isDev,
    parallel: parallel,
    throttle: throttle
});

/**
 * 默认的请求配置
 */
const DEFAULT_XHR_CFG = {
    timeout: 5000,
    withCredentials: false,
    parseContentType: 'json'
};
/**
 * 全局请求的 head 参数
 */
let BASE_HEAD_PARAMS = null;
/**
 * 设置全局请求的 head 参数
 *
 * @param param
 */
function setBaseHeadParams(params) {
    if (BASE_HEAD_PARAMS === null)
        BASE_HEAD_PARAMS = {};
    Object.assign(BASE_HEAD_PARAMS, params);
}
/**
 *
 * @param getOrDel
 * @param url
 * @param cb
 * @param params
 * @param cfg
 */
function getOrDel(getOrDel, url, cb, params, cfg = DEFAULT_XHR_CFG) {
    let xhr = initXhr(cfg);
    if (params != null) {
        if (url.indexOf('?') != -1)
            url += '&' + toParams(params);
        else
            url += '?' + toParams(params);
    }
    xhr.open(getOrDel.toUpperCase(), url, true);
    xhr.onreadystatechange = function () {
        responseHandle(this, cb, cfg);
    };
    if (BASE_HEAD_PARAMS) // 设置自定义请求头
        for (let key in BASE_HEAD_PARAMS)
            xhr.setRequestHeader(key, BASE_HEAD_PARAMS[key]);
    xhr.send();
}
/**
 *
 * @param method
 * @param url
 * @param cb
 * @param params
 * @param cfg
 */
function postOrPut(method, url, cb, params, cfg = DEFAULT_XHR_CFG) {
    let xhr = initXhr(cfg);
    xhr.open(method, url, true);
    xhr.onreadystatechange = function () {
        responseHandle(this, cb, cfg);
    };
    if (BASE_HEAD_PARAMS) // 设置自定义请求头
        for (let key in BASE_HEAD_PARAMS)
            xhr.setRequestHeader(key, BASE_HEAD_PARAMS[key]);
    // 此方法必须在 open() 方法和 send() 之间调用
    if (!cfg.contentType) // 如未设置，默认为表单请求
        xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    else
        xhr.setRequestHeader("Content-Type", cfg.contentType);
    let _params = typeof params != 'string' ? toParams(params) : params;
    if (_params)
        xhr.send(_params);
    else
        xhr.send();
}
/**
 *
 * @param url
 * @param cb
 * @param params
 * @param cfg
 */
function xhr_post_upload(url, cb, params, cfg = DEFAULT_XHR_CFG) {
    let xhr = initXhr(cfg);
    xhr.open('post', url, true);
    xhr.onreadystatechange = function () {
        responseHandle(this, cb, cfg);
    };
    if (BASE_HEAD_PARAMS) // 设置自定义请求头
        for (let key in BASE_HEAD_PARAMS)
            xhr.setRequestHeader(key, BASE_HEAD_PARAMS[key]);
    // 什么 Content-Type 都不设置
    xhr.send(params);
}
/**
 * XHR GET 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型是json；参数值会进行 URL 编码，最后附加到 QueryString 中
 * @param cfg       配置，可选的
 */
function xhr_get(url, cb, params, cfg = DEFAULT_XHR_CFG) {
    getOrDel('get', url, cb, params, cfg);
}
/**
 * XHR DELETE 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型是json；参数值会进行 URL 编码，最后附加到 QueryString 中
 * @param cfg       配置，可选的
 */
function xhr_del(url, cb, params, cfg = DEFAULT_XHR_CFG) {
    getOrDel('delete', url, cb, params, cfg);
}
/**
 * XHR POST 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型可以是字符串或 json；参数值会进行 URL 编码
 * @param cfg       配置，可选的
 */
function xhr_post(url, cb, params, cfg = DEFAULT_XHR_CFG) {
    postOrPut('post', url, cb, params, cfg);
}
/**
 * XHR PUT 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型可以是字符串或 json；参数值会进行 URL 编码
 * @param cfg       配置，可选的
 */
function xhr_put(url, cb, params, cfg = DEFAULT_XHR_CFG) {
    postOrPut('put', url, cb, params, cfg);
}
/**
 * 初始化 XHR
 *
 * @param cfg
 * @returns
 */
function initXhr(cfg) {
    let xhr = new XMLHttpRequest();
    if (cfg && cfg.timeout) {
        xhr.timeout = cfg.timeout;
        xhr.ontimeout = (e) => console.error('系统异常，XHR 连接服务超时');
    }
    if (cfg && cfg.withCredentials)
        xhr.withCredentials = true;
    return xhr;
}
/**
 * 错误处理
 *
 * @param xhr
 */
function errHandle(xhr) {
    let msg;
    if (xhr.status <= 400)
        msg = '请求参数错误或者权限不足。';
    else if (xhr.status <= 500)
        msg = '服务端异常。';
    else
        msg = `未知异常，HTTP code:${xhr.status}。`;
    if (!xhr.responseText)
        msg += " 服务端返回空的字符串！";
    console.error(msg, xhr.responseText);
}
/**
 * 响应处理
 *
 * @param xhr
 * @param cb
 * @param cfg
 */
function responseHandle(xhr, cb, cfg) {
    if (xhr.readyState == 4) {
        if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
            let text = xhr.responseText;
            let json;
            if (!text)
                console.warn('服务端没有返回任何字符串');
            switch (cfg.parseContentType) {
                case 'text':
                    break;
                case 'xml':
                    json = xhr.responseXML;
                    break;
                case 'json':
                default:
                    try {
                        json = JSON.parse(text);
                    }
                    catch (e) {
                        console.error('解析 JSON 时候发生错误，非法 JSON');
                        console.warn(e);
                    }
            }
            cb && cb(json, text);
        }
        else
            errHandle(xhr);
    }
}
/**
 * 对象转换为 URL 参数列表，用 & 分隔
 *
 * @param {Object} param JSON 对象
 * @returns URL 参数列表
 */
function toParams(param) {
    let result = "";
    for (let name in param) {
        if (typeof param[name] != "function")
            result += "&" + name + "=" + encodeURIComponent(param[name]);
    }
    return result.substring(1);
}
/**
 * 获取 QueryString 的某个参数
 *
 * @param val
 * @returns
*/
function getQuery(val) {
    const w = location.hash.indexOf('?');
    const query = location.hash.substring(w + 1);
    let vars = query.split('&');
    for (let i = 0; i < vars.length; i++) {
        const pair = vars[i].split('=');
        if (pair[0] == val)
            return pair[1];
    }
    return '';
}
function getPageList(self, listArray, callback) {
    return (j) => {
        if (j.status) {
            listArray.total = j.total;
            listArray.data = j.data;
            callback && callback();
        }
        else
            self.$Message.warning(j.message || '获取数据失败');
    };
}

var xhr = /*#__PURE__*/Object.freeze({
    __proto__: null,
    getPageList: getPageList,
    getQuery: getQuery,
    setBaseHeadParams: setBaseHeadParams,
    toParams: toParams,
    xhr_del: xhr_del,
    xhr_get: xhr_get,
    xhr_post: xhr_post,
    xhr_post_upload: xhr_post_upload,
    xhr_put: xhr_put
});

export { cookies as Cookies, dom as Dom, utils as Utils, xhr as Xhr };
