import { XhrConfig } from './xhr-config';
/**
 * 处理响应的回调函数
 */
type XhrCallback = (json: {}, text: string) => void;
/**
 * 设置全局请求的 head 参数
 *
 * @param param
 */
export declare function setBaseHeadParams(params: any): void;
/**
 *
 * @param url
 * @param cb
 * @param params
 * @param cfg
 */
export declare function xhr_post_upload(url: string, cb: XhrCallback, params: Document | XMLHttpRequestBodyInit, cfg?: XhrConfig): void;
/**
 * XHR GET 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型是json；参数值会进行 URL 编码，最后附加到 QueryString 中
 * @param cfg       配置，可选的
 */
export declare function xhr_get(url: string, cb: XhrCallback, params?: {}, cfg?: XhrConfig): void;
/**
 * XHR DELETE 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型是json；参数值会进行 URL 编码，最后附加到 QueryString 中
 * @param cfg       配置，可选的
 */
export declare function xhr_del(url: string, cb: XhrCallback, params?: {}, cfg?: XhrConfig): void;
/**
 * XHR POST 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型可以是字符串或 json；参数值会进行 URL 编码
 * @param cfg       配置，可选的
 */
export declare function xhr_post(url: string, cb: XhrCallback, params: string | {}, cfg?: XhrConfig): void;
/**
 * XHR PUT 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型可以是字符串或 json；参数值会进行 URL 编码
 * @param cfg       配置，可选的
 */
export declare function xhr_put(url: string, cb: XhrCallback, params: string | {}, cfg?: XhrConfig): void;
/**
 * 对象转换为 URL 参数列表，用 & 分隔
 *
 * @param {Object} param JSON 对象
 * @returns URL 参数列表
 */
export declare function toParams(param: any): string;
/**
 * 获取 QueryString 的某个参数
 *
 * @param val
 * @returns
*/
export declare function getQuery(val: string): string;
export declare function getPageList(self: any, listArray: any, callback?: Function): XhrCallback;
export {};
